"""
Page styling (sizes, colors, etc.) configuration for pages.
"""

HEADER_HEIGHT_PX = 60
NAVBAR_WIDTH_PX = 200

TYPE_TO_STATE = {
    "task-succeeded": {"message": "Succeeded", "color": "green"},
    "task-failed": {"message": "Failed", "color": "red"},
    "task-started": {"message": "In progress", "color": "blue"},
    "task-received": {"message": "Received", "color": "yellow"},
    "task-sent": {"message": "Sent", "color": "gray"},
    "default": {"message": "N/A", "color": "gray"},
}



"""Test celery monitor."""
import datetime
import os
from typing import Any, Literal

from celery import Celery

celery_app = Celery("tasks", broker=os.environ["CELERY_BROKER_URL"], backend=os.environ["CELERY_RESULT_BACKEND"])
celery_app.conf.timezone = "UTC"
celery_app.conf.worker_send_task_events = True
celery_app.conf.task_send_sent_event = True
celery_app.conf.task_track_started = True
celery_app.conf.result_extended = True


@celery_app.task(name="run_model", bind=True)
def run_model(self: Any, model_name: str) -> dict:
    """Run model"""
    return {"prediction": 0.85, "model_name": model_name}


@celery_app.task(name="add_one")
def add_one(my_number: int, when: datetime.datetime, flag: bool, name: str = "bar") -> int:
    """Adds one"""
    import time

    time.sleep(10)
    return my_number + 1


@celery_app.task(name="foo")
def foo(obj: dict, options: Literal["foo", "bar"]) -> dict:
    """Foo"""
    return obj



"""Task execution page"""

import os
from typing import Any

import dash_mantine_components as dmc
import requests  # type: ignore
from dash import set_props
from dash_extensions.enrich import ALL, MATCH, DashBlueprint, Input, Output, State, ctx, html
from loguru import logger

PORT = os.getenv("VEGGIE_PORT", 5000)

blueprint = DashBlueprint()


def create_input_component(task_name: str, param: dict) -> Any:
    """Creates a Dash input component based on the parameter type."""
    param_type = param["type"]
    default_value = param["default"]
    param_name = param["name"]

    if param_type == "str":
        return dmc.TextInput(
            label=param_name, id={"type": "input", "task": task_name, "param": param_name}, value=default_value or ""
        )
    elif param_type == "int":
        return dmc.NumberInput(
            label=param_name, id={"type": "input", "task": task_name, "param": param_name}, value=default_value or None
        )
    elif param_type == "datetime":
        return dmc.DateTimePicker(
            label=param_name,
            id={"type": "input", "task": task_name, "param": param_name},
            value=default_value or None,
            valueFormat="YYYY-MM-DD HH:mm:ss",
        )
    elif param_type == "bool":
        return dmc.Switch(label=param_name, id={"type": "input", "task": task_name, "param": param_name}, checked=False)
    else:
        return dmc.JsonInput(
            label=param_name,
            id={"type": "input", "task": task_name, "param": param_name},
            validationError="Invalid JSON",
            formatOnBlur=True,
            autosize=True,
            maxRows=4,
        )


def create_task_section(task_info: dict) -> Any:
    """
    Creates input components and run button for a task.
    """
    task_name = task_info["name"]
    parameters = task_info["parameters"]
    input_components = [create_input_component(task_name=task_name, param=param) for param in parameters]

    return dmc.Card(
        children=[
            dmc.Group([dmc.Text(task_name, fw=500)], justify="flex-start", mt="md", mb="xs"),
            dmc.Stack(children=input_components),
            dmc.Group(
                children=[dmc.Button(children="Run", id={"type": "run", "task": task_name}, mt="md")],
                justify="flex-end",
                mt="md",
                mb="xs",
            ),
        ],
        withBorder=True,
        shadow="sm",
        radius="md",
        w="70%",
    )


def layout() -> Any:
    """Creates layout for tasks page"""
    tasks = requests.get(f"http://localhost:{PORT}/api/tasks").json()
    tasks = sorted(tasks, key=lambda task: task["name"])

    task_layouts = []
    for task in tasks:
        task_card = create_task_section(task_info=task)
        task_layouts.append(task_card)

    return dmc.Container(
        children=dmc.Stack(align="center", mt="md", children=task_layouts + [html.Div(id="notifications-container")]),
        fluid=True,
    )


blueprint.layout = layout


@blueprint.callback(
    Output({"type": "run", "task": MATCH}, "dummy"),
    Input({"type": "run", "task": MATCH}, "n_clicks"),
    State({"type": "input", "task": ALL, "param": ALL}, "id"),
    State({"type": "input", "task": ALL, "param": ALL}, "value"),
    State({"type": "input", "task": ALL, "param": ALL}, "checked"),
    prevent_initial_call=True,
)
def handle_run(n_clicks: int, param_ids: list[dict], param_values: list, param_checked: list) -> Any:
    """Reads input values corresponding to a task and runs the task with Celery."""
    triggered_id = ctx.triggered_id
    task_name = triggered_id["task"]

    params = []
    for param_id, value, checked in zip(param_ids, param_values, param_checked, strict=True):
        if param_id["task"] == task_name:
            param_value = checked if isinstance(checked, bool) else value
            params.append({**param_id, "value": param_value})

    task_kwargs = {param["param"]: param["value"] for param in params}
    logger.info(task_kwargs)

    response = requests.post(
        url=f"http://localhost:{PORT}/api/tasks",
        json={"task_name": task_name, "task_params": task_kwargs},
        headers={"Content-Type": "application/json"},
    )
    if response.status_code != 200:
        logger.error(response.text)
        return None

    notification = dmc.Notification(
        title=f"Task {task_name} started",
        message=f"Task ID: {response.json()['task_id']}",
        action="show",
        position="bottom-right",
        autoClose=3000,
    )

    set_props(component_id="notifications-container", props={"children": [notification]})

    return None



"""REST API"""

from inspect import signature
from typing import Any, Callable, get_type_hints

from celery import Celery
from flask import Blueprint, Response, jsonify, request

from ..storage import Storage


def function_params_to_json(func: Callable) -> list[dict]:
    """
    Extracts function arguments, their types, and default values into JSON format.

    Args:
        func: The function to extract information from.

    Returns:
        A list of dictionaries, each representing one of the function's arguments.
    """
    # Get the function signature and type hints
    sig = signature(func)
    type_hints = get_type_hints(func)

    # Prepare JSON structure
    arguments = []
    for name, param in sig.parameters.items():
        param_type = type_hints.get(name, "Any")  # Use type hint if available, otherwise default to Any
        default_value = param.default if param.default is not param.empty else None  # Handle default values
        type_name = param_type.__name__ if hasattr(param_type, "__name__") else str(param_type)
        arguments.append({"name": name, "type": type_name, "default": default_value})
    return arguments


def create_api_blueprint(storage: Storage, celery_app: Celery) -> Blueprint:
    """Initializes Flask app"""
    bp = Blueprint(name="api", import_name=__name__)

    @bp.route("/events", methods=["GET"])
    def get_events() -> list[dict]:
        """Gets events"""
        return storage.get_events()

    @bp.route("/events/<string:event_id>", methods=["GET"])
    def get_single_event(event_id: str) -> Any:
        """Gets events"""
        event = storage.get_event_by_id(id=event_id)
        if event is None:
            return Response(status=404, response="Event not found")
        return event

    @bp.route("/tasks", methods=["GET"])
    def get_tasks() -> list[dict]:
        """Gets tasks"""
        user_defined_tasks = {name: task for name, task in celery_app.tasks.items() if not name.startswith("celery.")}
        return [
            {"name": name, "parameters": function_params_to_json(func=task)}
            for name, task in user_defined_tasks.items()
        ]

    @bp.route("/tasks", methods=["POST"])
    def send_task() -> Response:
        data = request.get_json()

        task_name = data.get("task_name")
        task_params = data.get("task_params")

        if task_name is None:
            return Response(status=400, response="Missing task_name")
        if task_params is None:
            return Response(status=400, response="Missing task_params")

        result = celery_app.send_task(task_name, kwargs=task_params)

        return jsonify({"task_id": result.task_id})

    return bp



"""
Header component of the dashboard.
"""

import dash_mantine_components as dmc
from dash_iconify import DashIconify

from ...webapp.dashboard import config

header_contents = dmc.Grid(
    align="center",
    justify="space-between",
    children=[
        dmc.GridCol(
            dmc.Group(
                [
                    dmc.Burger(id="burger", opened=False, hiddenFrom="sm", size="md"),
                    dmc.Anchor(
                        dmc.Group(
                            [DashIconify(icon="tdesign:broccoli", height=30), dmc.Text("Veggie", size="xl", fw=700)]
                        ),
                        href=config.APP_BASE_PATH,
                        refresh=False,
                        underline=False,
                        style={"text-decoration": "none", "color": "inherit"},
                    ),
                ]
            ),
            span="content",
        ),
        dmc.GridCol(
            children=[
                dmc.Group(
                    children=[
                        dmc.ActionIcon(
                            id="color-scheme-toggle",
                            children=[
                                dmc.Paper(DashIconify(icon="radix-icons:sun", width=20), lightHidden=True),
                                dmc.Paper(DashIconify(icon="radix-icons:moon", width=20), darkHidden=True),
                            ],
                            color="#FAB005",
                            size="lg",
                            variant="transparent",
                            radius="xl",
                        ),
                        dmc.Menu(
                            [dmc.MenuTarget(dmc.Anchor(children=dmc.Avatar(radius="xl"), href="#"))],
                            position="bottom-end",
                        ),
                    ]
                )
            ],
            span="content",
        ),
    ],
)



"""
Dash app
"""
import dash_mantine_components as dmc
from dash_extensions.enrich import DashProxy, Input, Output, State, dash, dcc, page_container

from ...webapp.dashboard import config, style
from ...webapp.dashboard.header import header_contents
from ...webapp.dashboard.navbar import navbar_contents
from ...webapp.dashboard.pages.not_found_404 import blueprint as not_found_blueprint
from ...webapp.dashboard.pages.status import blueprint as status_blueprint
from ...webapp.dashboard.pages.tasks import blueprint as tasks_blueprint

dash._dash_renderer._set_react_version("18.2.0")


app = DashProxy(
    suppress_callback_exceptions=True,
    use_pages=True,
    url_base_pathname=config.APP_BASE_PATH,
    external_stylesheets=[dmc.styles.DATES, dmc.styles.NOTIFICATIONS],
)
app.layout = dmc.MantineProvider(
    id="mantine-provider",
    forceColorScheme="dark",
    children=[
        dcc.Location(id="url"),
        dmc.NotificationProvider(),
        dmc.AppShell(
            id="appshell",
            header={"height": style.HEADER_HEIGHT_PX},
            transitionDuration=0,
            children=[
                dmc.AppShellHeader(
                    id="header",
                    children=header_contents,
                    style={"align-content": "center"},
                    zIndex=2000,
                    h=style.HEADER_HEIGHT_PX,
                    pl="md",
                    pr="md",
                ),
                dmc.AppShellNavbar(id="navbar", children=navbar_contents, w=style.NAVBAR_WIDTH_PX),
                dmc.AppShellMain(children=page_container, h="100%"),
            ],
            zIndex=1400,
        ),
    ],
)

status_blueprint.register(app, module="dash_apps.dashboard.pages.status", path="/")
tasks_blueprint.register(app, module="dash_apps.dashboard.pages.tasks", path="/tasks")
not_found_blueprint.register(app, module="dash_apps.dashboard.pages.not_found_404")


@app.callback(
    Output("mantine-provider", "forceColorScheme"),
    Input("color-scheme-toggle", "n_clicks"),
    State("mantine-provider", "forceColorScheme"),
    prevent_initial_call=True,
)
def switch_theme(n_clicks: int, theme: str) -> str:
    """Switches theme between light and dark based on theme toggle."""
    return "dark" if theme == "light" else "light"


@app.callback(Output("appshell", "navbar"), Input("burger", "opened"))
def on_burger_click(burger_opened: bool) -> dict:
    """Toggle navbar when burger menu is clicked."""
    navbar = {
        "width": style.NAVBAR_WIDTH_PX,
        "breakpoint": "sm",
        "collapsed": {"desktop": False, "mobile": not burger_opened},
    }
    return navbar



"""
Navbar for main app
"""

import dash_mantine_components as dmc
from dash import ALL, Input, Output, callback, callback_context
from dash_iconify import DashIconify

from veggie.webapp.dashboard import config

icon_width = 16

navlink_data = [
    {
        "id": "navlink-status",
        "label": "Status",
        "leftSection": DashIconify(icon="pajamas:status", width=icon_width),
        "variant": "light",
        "href": f"{config.APP_BASE_PATH}",
        "style": {"fontWeight": "100"},
    }
]

navbar_contents = dmc.ScrollArea(
    offsetScrollbars=True,
    type="scroll",
    style={"height": "100%", "padding-inline-end": "0"},
    children=dmc.Stack(
        gap=0,
        mt=30,
        children=[
            dmc.Button(
                "Refresh",
                id="refresh-repos-button",
                leftSection=DashIconify(icon="basil:refresh-solid", width=20),
                style={"display": "none"},
                mb="md",
                ml="md",
                mr="md",
            ),
            dmc.NavLink(
                id={"type": "navlink", "index": f"{config.APP_BASE_PATH}"},
                label="Status",
                leftSection=DashIconify(icon="pajamas:status", width=icon_width),
                variant="light",
                href=f"{config.APP_BASE_PATH}",
            ),
            dmc.NavLink(
                id={"type": "navlink", "index": f"{config.APP_BASE_PATH}tasks"},
                label="Start Tasks",
                leftSection=DashIconify(icon="fluent-mdl2:running", width=icon_width),
                variant="light",
                href=f"{config.APP_BASE_PATH}tasks",
            ),
        ],
    ),
)


@callback(Output({"type": "navlink", "index": ALL}, "active"), Input("url", "pathname"))
def update_navlinks(pathname: str) -> list[bool]:
    """Updates active navlink based on pathname."""
    return [control["id"]["index"] == pathname for control in callback_context.outputs_list]



"""
Dash app
"""

from typing import Any

from celery import Celery
from flask import Flask
from werkzeug.middleware.proxy_fix import ProxyFix

from ..storage import Storage
from ..webapp.api import create_api_blueprint
from ..webapp.dashboard.index import app as dash_app


def get_flask_app(storage: Storage, celery_app: Celery) -> Any:
    """Creates a Flask app and initializes Dash app inside it."""
    app = Flask(__name__)

    # This section is needed for url_for("foo", _external=True) to automatically
    # generate http scheme when this sample is running on localhost,
    # and to generate https scheme when it is deployed behind reversed proxy.
    # See also https://flask.palletsprojects.com/en/2.2.x/deploying/proxy_fix/
    app.wsgi_app = ProxyFix(app.wsgi_app, x_proto=1, x_host=1)  # type: ignore[method-assign]

    # Initialize Flask plugins
    dash_app.init_app(app)

    api_blueprint = create_api_blueprint(storage=storage, celery_app=celery_app)
    app.register_blueprint(blueprint=api_blueprint, url_prefix="/api")

    return app



"""Veggie monitor"""
from veggie.monitor import CeleryMonitor
from veggie.storage import SQLiteStorage

from example.tasks import celery_app

if __name__ == "__main__":
    celery_monitor = CeleryMonitor(celery_app=celery_app, storage=SQLiteStorage(path="events.db"))
    celery_monitor.start()



"""Setup script."""
from setuptools import find_packages, setup

with open("README.md", "r") as file:
    long_description = file.read()
    long_description = long_description.replace("\r", "")

setup(
    name="veggie",
    version="0.1.12",
    author="Theodore Tsitsimis",
    author_email="th.tsitsimis@gmail.com",
    description="A tool to monitor and execute Celery tasks",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=[
        "dash>=2.5.1",
        "dash-extensions>=1.0.19",
        "dash-mantine-components>=0.15.1",
        "dash-iconify",
        "loguru",
        "celery[redis]",
        "humanize",
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "License :: OSI Approved :: MIT License",
    ],
)



"""Status page"""
import datetime
import os
from typing import Any, Tuple

import dash_mantine_components as dmc
import humanize
import requests  # type: ignore
from dash_extensions.enrich import ALL, DashBlueprint, Input, Output, ctx, exceptions
from dash_iconify import DashIconify

from ....webapp.dashboard import style

PORT = os.getenv("VEGGIE_PORT", 5000)

blueprint = DashBlueprint()


def get_event_details_card(event: dict) -> Any:
    """Populates a card for the selected event"""
    state_message = style.TYPE_TO_STATE.get(event.get("type", "default"), {}).get("message")
    state_color = style.TYPE_TO_STATE.get(event.get("type", "default"), {}).get("color")

    sent_timestamp = (
        datetime.datetime.fromtimestamp(event.get("sent_timestamp", 0), tz=datetime.timezone.utc)
        if event.get("sent_timestamp")
        else None
    )
    received_timestamp = (
        datetime.datetime.fromtimestamp(event.get("received_timestamp", 0), tz=datetime.timezone.utc)
        if event.get("received_timestamp")
        else None
    )
    started_timestamp = (
        datetime.datetime.fromtimestamp(event.get("started_timestamp", 0), tz=datetime.timezone.utc)
        if event.get("started_timestamp")
        else None
    )
    succeeded_timestamp = (
        datetime.datetime.fromtimestamp(event.get("succeeded_timestamp", 0), tz=datetime.timezone.utc)
        if event.get("succeeded_timestamp")
        else None
    )
    failed_timestamp = (
        datetime.datetime.fromtimestamp(event.get("failed_timestamp", 0), tz=datetime.timezone.utc)
        if event.get("failed_timestamp")
        else None
    )

    if sent_timestamp is not None:
        sent_timestamp_str = sent_timestamp.strftime("%b %m %Y %H:%M:%S")
        natural_delta = humanize.naturaltime(datetime.datetime.now(tz=datetime.timezone.utc) - sent_timestamp)
    else:
        sent_timestamp_str = "N/A"
        natural_delta = "N/A"

    runtime = event.get("runtime")
    runtime = round(runtime, 2) if runtime is not None else None
    runtime_str = f"{runtime} s" if runtime is not None else "N/A"

    return dmc.Stack(
        m="md",
        children=[
            dmc.Text(f"{sent_timestamp_str} ({natural_delta})", c="dimmed"),
            dmc.Group(
                gap="xs",
                justify="flex-start",
                children=[dmc.Text("Task ", c="dimmed", fw=700), dmc.Text([event.get("name")], fw=700)],
            ),
            dmc.Group(
                gap="lg",
                children=[
                    dmc.Stack(children=[dmc.Text("Status", c="dimmed"), dmc.Badge(state_message, color=state_color)]),
                    dmc.Stack(children=[dmc.Text("Runtime", c="dimmed"), dmc.Text(runtime_str)]),
                    dmc.Stack(children=[dmc.Text("UUID", c="dimmed"), dmc.Text(event.get("uuid"))]),
                ],
            ),
            dmc.Divider(variant="solid"),
            dmc.Stack(
                children=[
                    dmc.Text("Details ", fw=700),
                    dmc.Table(
                        withColumnBorders=False,
                        withTableBorder=False,
                        withRowBorders=False,
                        children=[
                            dmc.TableTbody(
                                children=[
                                    dmc.TableTr(
                                        children=[
                                            dmc.TableTd(dmc.Text("Sent timestamp", c="dimmed")),
                                            dmc.TableTd(
                                                sent_timestamp.strftime("%b %m %Y %H:%M:%S")
                                                if sent_timestamp
                                                else "N/A"
                                            ),
                                        ]
                                    ),
                                    dmc.TableTr(
                                        children=[
                                            dmc.TableTd(dmc.Text("Received timestamp", c="dimmed")),
                                            dmc.TableTd(
                                                received_timestamp.strftime("%b %m %Y %H:%M:%S")
                                                if received_timestamp
                                                else "N/A"
                                            ),
                                        ]
                                    ),
                                    dmc.TableTr(
                                        children=[
                                            dmc.TableTd(dmc.Text("Started timestamp", c="dimmed")),
                                            dmc.TableTd(
                                                started_timestamp.strftime("%b %m %Y %H:%M:%S")
                                                if started_timestamp
                                                else "N/A"
                                            ),
                                        ]
                                    ),
                                    dmc.TableTr(
                                        children=[
                                            dmc.TableTd(dmc.Text("Succeeded timestamp", c="dimmed")),
                                            dmc.TableTd(
                                                succeeded_timestamp.strftime("%b %m %Y %H:%M:%S")
                                                if succeeded_timestamp
                                                else "N/A"
                                            ),
                                        ]
                                    ),
                                    dmc.TableTr(
                                        children=[
                                            dmc.TableTd(dmc.Text("Failed timestamp", c="dimmed")),
                                            dmc.TableTd(
                                                failed_timestamp.strftime("%b %m %Y %H:%M:%S")
                                                if failed_timestamp
                                                else "N/A"
                                            ),
                                        ]
                                    ),
                                    dmc.TableTr(
                                        children=[
                                            dmc.TableTd(dmc.Text("Hostname", c="dimmed")),
                                            dmc.TableTd(event.get("hostname")),
                                        ]
                                    ),
                                    dmc.TableTr(
                                        children=[
                                            dmc.TableTd(dmc.Text("args", c="dimmed")),
                                            dmc.TableTd(event.get("args")),
                                        ]
                                    ),
                                    dmc.TableTr(
                                        children=[
                                            dmc.TableTd(dmc.Text("kwargs", c="dimmed")),
                                            dmc.TableTd(event.get("kwargs")),
                                        ]
                                    ),
                                    dmc.TableTr(
                                        children=[
                                            dmc.TableTd(dmc.Text("result", c="dimmed")),
                                            dmc.Code(event.get("result")),
                                        ]
                                    ),
                                    dmc.TableTr(
                                        children=[
                                            dmc.TableTd(dmc.Text("pid", c="dimmed")),
                                            dmc.TableTd(event.get("pid")),
                                        ]
                                    ),
                                    dmc.TableTr(
                                        children=[
                                            dmc.TableTd(dmc.Text("parent_id", c="dimmed")),
                                            dmc.TableTd(event.get("parent_id")),
                                        ]
                                    ),
                                    dmc.TableTr(
                                        children=[
                                            dmc.TableTd(dmc.Text("retries", c="dimmed")),
                                            dmc.TableTd(event.get("retries")),
                                        ]
                                    ),
                                    dmc.TableTr(
                                        children=[
                                            dmc.TableTd(dmc.Text("routing_key", c="dimmed")),
                                            dmc.TableTd(event.get("routing_key")),
                                        ]
                                    ),
                                ]
                            )
                        ],
                    ),
                ]
            ),
        ],
    )


def get_table_rows() -> list[Any]:
    """Retrieves events and creates table body"""
    response = requests.get(f"http://localhost:{PORT}/api/events")
    events = response.json()

    rows = []
    for event in events:
        state_message = style.TYPE_TO_STATE.get(event.get("type"), {}).get("message", "")
        state_color = style.TYPE_TO_STATE.get(event.get("type"), {}).get("color", "gray")

        row = dmc.TableTr(
            children=[
                dmc.TableTd(
                    dmc.ActionIcon(
                        DashIconify(icon="mingcute:more-2-line", width=20),
                        size="md",
                        variant="transparent",
                        id={"type": "event-details-button", "uuid": event["uuid"]},
                    )
                ),
                dmc.TableTd(event["name"]),
                dmc.TableTd(dmc.Badge(state_message, color=state_color)),
                dmc.TableTd(
                    humanize.naturaltime(
                        datetime.datetime.now(tz=datetime.timezone.utc)
                        - datetime.datetime.fromtimestamp(event["received_timestamp"], tz=datetime.timezone.utc)
                    )
                ),
            ]
        )
        rows.append(row)

    return rows


def layout() -> Any:
    """Creates layout for status page"""
    return dmc.Container(
        children=[
            dmc.Stack(
                align="flex-start",
                gap="md",
                mt="md",
                children=[
                    dmc.Button(
                        id="status-update-button",
                        children="Refresh",
                        leftSection=DashIconify(icon="material-symbols:refresh", width=20),
                    ),
                    dmc.Table(
                        id="task-status-table",
                        withColumnBorders=False,
                        withTableBorder=True,
                        children=[
                            dmc.TableThead(
                                dmc.TableTr(
                                    [
                                        dmc.TableTh(""),
                                        dmc.TableTh("Task"),
                                        dmc.TableTh("Status"),
                                        dmc.TableTh("Received"),
                                    ]
                                )
                            ),
                            dmc.TableTbody(id="task-status-table-body", children=get_table_rows()),
                        ],
                    ),
                ],
            ),
            dmc.Drawer(id="event-details-drawer", position="right", size="60%", opened=False),
        ],
        fluid=True,
    )


blueprint.layout = layout


@blueprint.callback(
    Output("task-status-table-body", "children"), Input("status-update-button", "n_clicks"), prevent_initial_call=True
)
def update_table(n_clicks: int) -> list[Any]:
    """Updates status table"""
    return get_table_rows()


@blueprint.callback(
    Output("event-details-drawer", "opened"),
    Output("event-details-drawer", "children"),
    Input({"type": "event-details-button", "uuid": ALL}, "n_clicks"),
    prevent_initial_call=True,
)
def open_event_details(n_clicks: list[int | None]) -> Tuple[bool, Any]:
    """Opens event details drawer."""
    if all(item is None for item in n_clicks):
        raise exceptions.PreventUpdate

    triggered_id = ctx.triggered_id
    task_id = triggered_id["uuid"]

    response = requests.get(f"http://localhost:{PORT}/api/events/{task_id}")
    event = response.json()

    return True, get_event_details_card(event=event)
